##########################################################
#
# boaSaboca.py
#
# Boa script for pipeline reduction of SABOCA data
#
# Created: 2007/9/08 (copy of older scripts)
#
# Author: F. Schuller
#
# who       when        what
# -----------------------------------------------------
# mdumke    2010-05-01  Added invertSomeChannels 
#                           & getSabocaInvert functions
# aweiss    2010-05-15  Added calcsensitivity
# fschuller 2013-03-23  Added getSabocaVtoJy
##########################################################

# Read LABOCA specific definitions
execfile(os.getenv('BOA_HOME_LABOCA')+'/../saboca/saboca-cabling.py')

from boa.fortran import fStat
from boa.Utilities import Timing
myTiming = Timing()



def getSabocaRCP(mjdref):
	
	try:
 		f = file(os.path.join(os.getenv('BOA_HOME_LABOCA'), '../saboca/Saboca-rcps.dat'))
	except IOError:
		self.__MessHand.error("could not open Saboca-rcps.dat")	
		return

	param = f.readlines()	
	f.close()

	mjdstart = []
	mjdstop = []
	rcp=[]
	for i in range(len(param)):
 		if param[i][0] not in ['!','#','n']:    # skip comments
                	tmp = string.split(param[i])
                	mjdstart.append(string.atof(tmp[0]))
                	mjdstop.append(string.atof(tmp[1]))
			sr=tmp[2]
                	rcp.append(sr)

	mjdstart = array(mjdstart)
    	mjdstop =  array(mjdstop)
    	
	rcpname= -1

	for i in range(len(mjdstart)):
		low = mjdstart[i]
		high = mjdstop[i]
		if low <= mjdref and mjdref < high:
			rcpname = rcp[i]

	return rcpname


def getSabocaVtoJy(mjdref):
	
	try:
 		f = file(os.path.join(os.getenv('BOA_HOME_SABOCA'), 'Saboca-VtoJy.dat'))
	except IOError:
		self.__MessHand.error("could not open Saboca-VtoJy.dat")	
		return

	param = f.readlines()	
	f.close()

	result = -1
	for i in range(len(param)):
		if param[i][0] not in ['!','#','n']:    # skip comments
                	tmp = string.split(param[i])
                	mjdstart = float(tmp[0])
                	mjdstop  = float(tmp[1])
			if mjdstart <= mjdref and mjdref < mjdstop:
				result = float(tmp[2])
	return result


def getSabocaCross(mjdref):
	
    try:
        f = file(os.path.join(os.getenv('BOA_HOME_LABOCA'), '../saboca/Saboca-cross.dat'))
    except IOError:
        self.__MessHand.error("could not open Saboca-cross.dat")	
        return

    param = f.readlines()	
    f.close()

    mjdstart = []
    mjdstop = []
    cr=[]
    for i in range(len(param)):
 		if param[i][0] not in ['!','#','n']:    # skip comments
                	tmp = string.split(param[i])
                	mjdstart.append(string.atof(tmp[0]))
                	mjdstop.append(string.atof(tmp[1]))
			sr=tmp[2]
                	cr.append(sr)

    mjdstart = array(mjdstart)
    mjdstop =  array(mjdstop)
    	
    cross= []
    crossreturn = []
    for i in range(len(mjdstart)):
		low = mjdstart[i]
		high = mjdstop[i]
		if low <= mjdref and mjdref < high:
			cross = cr[i]


    tmp=cross.split(',')
    for a in tmp:
        crossreturn.append(int(a))

    return crossreturn

def invertSomeChannels(iclist=[]):
    for i in range(len(iclist)):
        num=iclist[i]-1
        data.Data[:,num] *= array(-1,'f')

def getSabocaInvert(mjdref):
	
    try:
        f = file(os.path.join(os.getenv('BOA_HOME_LABOCA'), '../saboca/Saboca-invert.dat'))
    except IOError:
        self.__MessHand.error("could not open Saboca-invert.dat")	
        return

    param = f.readlines()	
    f.close()

    mjdstart = []
    mjdstop = []
    inv=[]
    for i in range(len(param)):
 		if param[i][0] not in ['!','#','n']:    # skip comments
                	tmp = string.split(param[i])
                	mjdstart.append(string.atof(tmp[0]))
                	mjdstop.append(string.atof(tmp[1]))
			sr=tmp[2]
                	inv.append(sr)

    mjdstart = array(mjdstart)
    mjdstop =  array(mjdstop)
    	
    invert = []
    invertreturn = []
    for i in range(len(mjdstart)):
	    low = mjdstart[i]
	    high = mjdstop[i]
	    if low <= mjdref and mjdref < high:
	        invert = inv[i]


    tmp=invert.split(',')
    for a in tmp:
        invertreturn.append(int(a))
    if invertreturn == [0]:
        invertreturn = []

    return invertreturn


def calcsensitivity():
        from boa.fortran import fStat
        usechan = data.BolometerArray.checkChanList([])
        nrbolos = len(usechan)
        weight()
        rms = array((data.getChanListData('rms')),'f')
        adt=(data.ScanParam.get('deltat'))
        dt=fStat.f_median(adt)
        sensitivity=rms*sqrt(dt)*1000.
        array_sensitivity = sum(sensitivity/(rms**2))/sum(1./(rms**2))
        return  array_sensitivity
